/*
 * Copyright 2015 The gRPC Authors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.yuandian.dataflow.grpc;

import com.google.protobuf.Any;
import com.yuandian.dataflow.proto.Base;
import com.yuandian.dataflow.proto.CollectPacketsServerGrpc;
import com.yuandian.dataflow.proto.msgtype.AppFlowOuterClass;
import com.yuandian.dataflow.proto.msgtype.BacktrackingFlowOuterClass;
import io.grpc.Server;
import io.grpc.ServerBuilder;
import io.grpc.stub.StreamObserver;

import java.io.IOException;
import java.util.concurrent.TimeUnit;
import java.util.logging.Logger;

/**
 * Server that manages startup/shutdown of a {@code Greeter} server.
 */
public class CollectPacketsServer extends CollectPacketsServerGrpc.CollectPacketsServerImplBase {
    private static final Logger logger = Logger.getLogger(CollectPacketsServer.class.getName());

    @Override
    public void getPackets(Base.Request request, StreamObserver<Base.Response> responseObserver) {
        for (int i = 0; i < 1; i++) {
            responseObserver.onNext(Base.Response.newBuilder()
                    .setCode(200)
                    .setMessage("测试一波")
                    .addPackets(
                            Any.pack(
                                    BacktrackingFlowOuterClass.BacktrackingFlow.newBuilder()
                                            .setTableId(10086)
                                            .build()
                            )
                    )
                    .build());
            System.out.println("CollectPacketsServer: " + i);
        }
        responseObserver.onCompleted();
    }

    //private Server server;

    /**
     * Main launches the server from the command line.
     */
    public static void main(String[] args) throws IOException, InterruptedException {
        Server server = ServerBuilder.forPort(60017)
                .addService(new CollectPacketsServer())
                .build()
                .start();
        Runtime.getRuntime().addShutdownHook(new Thread() {
            @Override
            public void run() {
                // Use stderr here since the logger may have been reset by its JVM shutdown hook.
                System.err.println("*** shutting down gRPC server since JVM is shutting down");
                try {
                    server.shutdown().awaitTermination(30, TimeUnit.SECONDS);
                } catch (InterruptedException e) {
                    e.printStackTrace(System.err);
                }
                System.err.println("*** server shut down");
            }
        });
        server.awaitTermination();
    }
}
